package com.example.demo.supply.services;

import com.example.demo.supply.models.Product;
import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityNotFoundException;
import jakarta.persistence.PersistenceContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;

@Service
public class ProductService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Product addProduct(String name, float cost){
        if(!StringUtils.hasText(name) || cost <= 0)
            throw new IllegalArgumentException("Invalid name and/or cost");

        final Product product = new Product(name, cost);
        em.persist(product);
        return product;
    }

    @Transactional(readOnly = true)
    public Product findProduct(Long id) {
        final Product product = em.find(Product.class, id);
        if (product == null) {
            throw new EntityNotFoundException(String.format("Product with id [%s] is not found", id));
        }
        return product;
    }

    @Transactional(readOnly = true)
    public List<Product> findAllProducts() {
        return em.createQuery("select p from Product p", Product.class).getResultList();
    }

    @Transactional
    public Product updateProduct(Long id, String name, float cost) {
        if (!StringUtils.hasText(name) || cost <= 0)
            throw new IllegalArgumentException("Invalid name and/or cost");

        final Product currentProduct = findProduct(id);
        currentProduct.setName(name);
        currentProduct.setCost(cost);
        return em.merge(currentProduct);
    }

    @Transactional
    public Product deleteProduct(Long id) {
        final Product currentProduct = findProduct(id);
        em.remove(currentProduct);
        return currentProduct;
    }
}
